@@ -112,6 +112,8 @@ static void gripe_rwopen_failed (void)
 {
 	if (errno == EACCES || errno == EROFS)
 		debug ("database %s is read-only\n", database);
+	else if (errno == EAGAIN || errno == EWOULDBLOCK)
+		debug ("database %s is locked by another process\n", database);
 	else {
 #ifdef MAN_DB_UPDATES
 		if (!quiet)
@@ -182,22 +184,21 @@ void test_manfile (const char *file, const char *path)
 				return;
 			}
 		} else {
-			struct stat physical;
 			char *abs_filename;
-			
+
 			/* see if the cached file actually exists. It's 
 			   evident at this point that we have multiple 
 			   comp extensions */
 			abs_filename = make_filename (path, NULL,
 						      exists, "man");
-			debug ("test_manfile(): stat %s\n", abs_filename);
-			if (stat (abs_filename, &physical) == -1) {
+			if (!abs_filename) {
 				if (!opt_test)
 					dbdelete (manpage_base, exists);
 			} else {
 				gripe_multi_extensions (path, exists->sec,
 							manpage_base,
 							exists->ext);
+				free (abs_filename);
 				free_mandata_struct (exists);
 				free (manpage);
 				return;
@@ -546,11 +547,20 @@ void update_db_time (void)
 	/* we know that this should succeed because we just updated the db! */
 	dbf = MYDBM_RWOPEN (database);
 	if (dbf == NULL) {
-#ifdef MAN_DB_UPDATES
-		if (!quiet)
-#endif /* MAN_DB_UPDATES */
-			error (0, errno, _("can't update index cache %s"),
+		if (errno == EAGAIN || errno == EWOULDBLOCK)
+			/* Another mandb process is probably running.  With
+			 * any luck it will update the mtime ...
+			 */
+			debug ("database %s is locked by another process\n",
 			       database);
+		else {
+#ifdef MAN_DB_UPDATES
+			if (!quiet)
+#endif /* MAN_DB_UPDATES */
+				error (0, errno,
+				       _("can't update index cache %s"),
+				       database);
+		}
 		free (MYDBM_DPTR (content));
 		return;
 	}
